------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local db = {}

local tins,concat,format = table.insert,table.concat,string.format

function db:run(recno,subname,subform)
  local dbg = {}
  local cli = self.conf.cli and {menu = {}} or nil
  if (tonumber(recno) or 0) <= 0 and not cli then return end
  local dname = self.model
  local dform = self.def.view
  local sub = ""
  local out,js = {},self.conf.js

  if subname then
    dname = subname
    dform = subform
    sub = dname
  end
  self.util.apps(self,dform) --clean on apps

  local row,nrow,qry
  if (tonumber(recno) or 0) > 0 then
    qry = self.util:mkqry(recno,dform.mod or dname,dform)
    local stmt = self.conn:prepare(qry)

    assert(stmt,concat{self.conn:errmsg(),":",qry})

    if stmt:step() == self.dbm.ROW then
        row = stmt:get_values()
        nrow = stmt:get_named_values()
    end
    stmt:finalize()

    if (nrow and nrow.user and not self:access(dform.mod or dname,recno,nrow.user,"view")) 
          or not nrow then
      return concat{lang["No access read"] or concat{"No access read"}}
    end
  end
  local guest = self.user.guest
  local ii,tr,fld = 2,0,2
  for k,v in ipairs(dform) do
    local vid,nm = v[1],v[2]
    local name = ""
    local val = cli and params[sub..vid] or nil
    local value = val or ""
    if row and v.ht ~= "SHOW" and nm then
      if row[ii] then
        val = cli and val or row[ii]
        value = v.ht ~= "HTML" and v.ht ~= "PAGE" and self.util:escape(val) or val
      end
      ii = ii + 1
    end
    if not nm then
      ii = ii + 1
    elseif v.ht == "CHECKBOX" then
      local yes = lang.Yes or "Yes"
      local no = lang.No or "No"
      if v.slider and type(v.slider) == 'table' then
        yes = v.slider[2]
        no = v.slider[1]
      end
      value = (tonumber(value) or 0) == 1 and yes or no
    elseif v.ht == "REF" and v.rt then
      local mod = v.rt
      if mod:match("%.") then
        local app = mod:match("([^%.]+)")
        if not self.conf.apps[app] then nm = nil end --hide
      end

      if (tonumber(value) or 0) > 0 and nm then
        if not (mod or ''):match('%.') then --todo base
          if (vid or ''):match('%.') and not self.conf.site[self.app] then
            mod = concat{self.app,'.',vid:match('(.*)%.')}
          elseif mod and self.conf.apps[self.app] and self.conf.apps[self.app].mods and self.conf.apps[self.app].mods[mod] then
            mod = concat{self.app,'.',mod}
          end
        end
        local nref = value
        local substmt = self.conn:prepare(self.util:mkqry(value,v.rt,v.rf or {{"name"}}))
        if substmt and substmt:step() == self.dbm.ROW then
          local subrow = substmt:get_values()
          value = ""
          for kk,vv in ipairs(subrow) do
            if kk ~= 1 and vv and vv ~= "" then
              value = concat{value,kk > 2 and " " or "",vv}
            end
          end
          value = self.util:escape(value)
          substmt:finalize()
        else
          value = "?" .. value
        end

        local str = " "
        if (tonumber(recno) or 0) > 0 then str = value end
        if not str or str == '' then str = nm end -- empty name
        if not cli then
          value = concat{self:link(mod,str,{act='show',recno=nref}),"&#160;",
            self:link('popup',format("<img border='0' src='%s/site/face/img/plus.png'/>",self.conf.path),
              {mod=mod,act='show',recno=nref,title=str})}
        end
      end
    elseif v.ht == "SELECT" then
      if not v.rf then v.rf = {{'name','Name'}} end
      if not v.rt then
        for kk,vv in ipairs(type(v.rf) == "table" and v.rf or self.conf[v.rf] or {}) do
          if tostring(value) == tostring(vv[1]) then value = lang[vv[2]] or vv[2] or vv[1]
            break
          end
        end
      elseif cli and v.rf[1] and v.rf[1].ht == "SHOW" then
        if self.def.field and self.def.field[vid] then
          v.rf = self.def.field[vid](self,nrow) or {}
          v.rf.ht = "SHOW"
        end
      elseif (tonumber(value) or 0) > 0 then
        local mod = v.rt
        local qry = self.util:mkqry(value,v.rt,v.rf)
        local substmt = self.conn:prepare(qry)
        if not substmt then
          tins(out,concat{self.conn:errmsg(),":",qry})
        else
          for subrow in substmt:rows() do
            local names = {}
            local subrec = 0
            for kk,vv in ipairs(subrow) do
              if kk == 1 then subrec = vv else names[#names+1]=self.util:escape(vv or "") end
            end
            if #names > 0 then
              value = concat(names,' ')
              local mod = v.rt

              if not (mod or ''):match('%.') then --todo base
                if (vid or ''):match('%.') and not self.conf.site[self.app] then
                  mod = concat{self.app,'.',vid:match('(.*)%.')}
                elseif mod and self.app and self.conf.apps[self.app] and self.conf.apps[self.app].mods and self.conf.apps[self.app].mods[mod] then
                  mod = concat{self.app,'.',mod}
                end
              end
              if not value or value == '' then value = nm end -- empty name

              if not cli then value = self:link(mod,value,{act='show',recno=subrec}) end
            end
          end
          substmt:finalize()
        end
      end
    elseif v.ht == "MULTI" then
      local list = {}
      for item in string.gmatch(value or "","[_%w]+") do
        tins(list,(lang[item] or item))
      end
      value = concat(list,cli and "," or "<br/>")
    elseif v.ht == "SHOW" and recno then
      value = ""
      if self.def.field and self.def.field[vid] and not v.help then
        value = self.def.field[vid]
        if type(value) == 'function' then value = value(self,nrow) or "" end
      end
    elseif v.ht == "OBJECT" then
      local nrec = tonumber(value) or 0
      for rowid in self.conn:urows('select rowid from ' .. v.rt .. ' where ' .. v[1] .. ' = ' .. recno) do
          nrec = rowid
      end
      if nrec > 0 or cli then
        value = self:run(nrec,v.rt,v.rf)
        if cli and nrec > 0 then params[v.rt.."_recno"] = nrec end
      else
        value = ""
      end
    elseif v.ht == "WWW" then
      if not cli then value = concat{"<a href='",value,"'>",value,"</a>"} end
    elseif v.ht == "FILE" then
      if  (v.img or v.camera) and not cli then
        value = concat{'<div style="padding:5px;max-width:800px;text-align:left;" >',
                      '<img src="',self.conf.idx,'?mod=',dname,'&act=download&recno=',nrow.rowid,
                                                '&datafld=',v.img or v.camera,'&namefld=',v[1],
                                               '" style="max-width:250px;max-height:250px;" alt=""',
                       '/>',
                      '</div>',
                    }
      end
    elseif v.ht == "EMAIL" then
      if value:find("@") and not cli then 
        value = self:link("mails",value,{act='new',email=value})
      end
    elseif v.ht == "PW" then
      value = value:gsub(".","*")
    elseif v.ht == "TEXTAREA" or v.ht == "HTML" then
      if not cli then value = concat{"<pre wrap='wrap'>",value,"</pre>"} end
    --elseif v.ht == "PAGE" then
    --  if not cli then value = concat{"<div id='",v[1] .. "PAGE","'>",value,"</div>"} end
    elseif v.ht == "DATETIME" then
      value = math.abs(tonumber(value) or 0) > 0 and os.date(self.conf.time or "%Y-%m-%d %H:%M",tonumber(value)) or val
    elseif v.ht == "DATE" then
      value = math.abs(tonumber(value) or 0) > 0 and os.date(self.conf.date or "%Y-%m-%d",tonumber(value)) or val
    elseif v.ht == "LIST" then
      local list = {}
      for item in string.gmatch(value or "","[^;]+") do tins(list,(lang[item] or item)) end
      value = concat(list,"<br/>")
    elseif v.ht == "MATRIX" and not cli then
      local tb = concat{[[<table class='inner' border='1'
        frame='void' rules='none' cellpadding='1' cellspacing='1'><tr><th>]],
        lang.Name or "Name","</th>"}
      for k,v in ipairs(v.rx) do tb=concat{tb,"<th>",lang[v[2]] or v[2],"</th>"} end
      tb=concat{tb,"<tr/>"}
      if v.ry.rt and v.ry.rf then
        local qry = concat{"select ",v.ry.rf[1][1],",",v.ry.rf[1][2]," from ",v.ry.rt,
          v.ry.wh and concat{" where ",v.ry.wh} or ""}
        v.ry.rf = {}
        for id,name in self.conn:urows(qry) do v.ry.rf[#v.ry.rf+1] = {id,name} end
      end
      for _,row in ipairs(v.ry.rf or {}) do
        local id,name,grp = row[1],row[2],row[3]
        local class = grp and 'line2' or 'line1'
        tb=concat{tb,"<tr class='",class,"'><td>",grp and '<b>' or '',
                    lang[name] or name,grp and '</b>' or '',"</td>"}
        local y,x = string.match(value or "","("..id.."):(%a*)")
        local empty = (not value or value == '' or not y)
        for k,v in ipairs(v.rx) do
          local val = (empty or (x and string.find(x,v[1]))) and 1 or 0
          tb=concat{tb,"<td align='middle'>",val,"</td>"}
        end
        tb=concat{tb,"</tr>"}
      end
      value = concat{tb,"</table>"}
    end

    if v.ht == "OBJECT" and nm then
      tins(out,value)
      if cli and type(value) == "table" then
        for k,v in ipairs(value) do tins(cli,v) end
      end
    elseif nm and not (vid == 'user' and guest) then
      if name == "" then name = lang[nm] or nm end
      if tr == 0 then tins(out,"<tr>") end
      tins(out,format([[<td class='desc' valign='top' align='%s' nowrap='nowrap' width='15%%'>%s</td>
                        <td class='line1' align='left' valign='top' %s>%s</td>]],v.align,
                        name,v.opt or '',value or ""))
      tr = tr + 1
      local columns = dform.columns or self.conf.view.cols or 1
      if tr == columns then tins(out,"</tr>") tr = 0 end
      if cli then tins(cli,{name,value,v or {},sub,val}) end
    end
  end -- for
  if nrow and nrow.user and not guest then
    local created = os.date(self.conf.time or "%Y-%m-%d %H:%M",tonumber(nrow.created or 0))
    local edited = os.date(self.conf.time or "%Y-%m-%d %H:%M",tonumber(nrow.edited or 0))
    local create = concat{created," ",nrow.creator}
    local edit = ''
    if created ~= edited then edit = concat{not cli and '<br/>' or '',edited," ",nrow.editor} end
    if not cli then
      tins(out,format([[<td class='desc' valign='top' align='%s' nowrap='nowrap' width='15%%'>%s</td>
      <td class='line1' align='left' valign='top'>%s%s</td>]],dform[1] and dform[1].align or "left",lang.Change or "Change",
      create,edit))
    else tins(cli,{"Change",edit}) end
  end

  if tr ~= 0 then tins(out,"</tr>") end
  local prn = params.prn
  if subname then if cli then return cli else return concat(out) end end

  local toprn = (js and not prn) and
       format([[<div align='right'><a class='nodeco' href="%s">
       <img border='0' src='%s/site/face/img/print.gif'/></a></div>]],
       self:link('popup',nil,{mod=dname,recno=recno,prn=1}),self.conf.path) or ""
  local acts = {}
--
  local moves = {--{self:link(self.model,lang.Prev or 'Prev',{act='prev',recno=recno-1},'submit'),
                 -- '&#160;',
                 --self:link(self.model,lang.Next or 'Next',{act='next',recno=recno+1},'submit')}
                 --self:link(self.model,lang.Next or 'Next',{act='next',recno=recno+1},'submit')}
  }
  for k,v in ipairs(dform.acts or {}) do
    moves[#moves+1] = self:link(v[1],v[2],v[3])
  end
--
  local btns = {} --form buttons

  if not prn and not dform.view and (nrow and (not nrow.user or self:access(dname,nil,nrow.user,"edit"))) or not nrow then
    local edit = ""
    if not dform.readonly then
      edit = self:link(dname,lang.Edit or "Edit",{act='edit',recno=recno},"submit","edfrm")
      btns[#btns + 1] = edit
    end
    local del = ''
    if self:access(dname,nil,nrow and nrow.user,"delete") and not dform.readonly and not dform.nodelete then
      del = self:link(dname,lang.Delete or "Delete",{act='delete',recno=recno},"submit","edfrm")
      btns[#btns + 1] = del
    end
    if self.conf.js then
      edit,del = '',''
    end
    tins(acts,format([[<table border='0' width='100%%' cellspacing='1'>
    <tr><td align='center'>%s %s %s</td></tr></table>]],
    edit,del,concat(moves)))
  end

  if self.def.actions and recno then
    for _,act in ipairs(self.def:actions(nrow,recno) or {}) do
      btns[#btns + 1] = act
    end
  end
--
  local menu = nrow and self.defs[dname].menu and self.defs[dname]:menu(nrow) or {}

  dform.list = self.util.rels(self,dname,dform.list)
  debug.log('LIST:',dname)

  for i,row in ipairs(dform.list or {}) do --name,ls,idx,app
    debug.log('ITEM',row[1])
    local id,rel,qry = row[1],row[2],{act="new"}
    for k,v in pairs(rel.link or {}) do
      if k == "id" then 
        if type(rel.link.id) == 'string' then
          if row[2] and row[2].edit then -- redir to parent after child save
            qry.pmod = dname
            qry.prec = recno
          end
          qry[rel.link.id] = recno
        elseif type(rel.link.id) == 'table' then
          for _,fld in ipairs(rel.link.id) do
            qry[fld] = recno
          end
        end
      elseif k == "mod" then qry[rel.link.mod] = dname
      elseif k == "ls" then
        dform.list[i][2] = v
      elseif nrow then  qry[k] = nrow[v] end
    end

    if self:access(id,nil,self.user.id,"new",row[4]) and not rel.show then --todo list.show
      for key,val in pairs(menu[id] or {}) do -- if table id in menu then get defaults from
        qry[key] = nrow[val]
      end
      tins(menu,{id,rel.name or id,qry,ext=true}) -- for show not ext in cli
    end
  end

  for k,v in pairs(menu) do --todo for miss index (k = 1,3,4)
    if not v[1] then
    elseif string.find(v[1],"javascript") then
      menu[k] = format([[<li><a class='nodeco' href="%s">%s</a></li>]],
        v[1],lang[v[2]] or v[2])
    elseif self.user.admin or not string.match(self.user.pages or "",concat{v[1],":0"}) then
      if self.util.apps(self,v[1]) then
        if cli then
          tins(cli.menu,v)
        else
          if type(v[4]) == 'table' and not v[4].css then --opts
            v[4].css = 'nodeco'
          elseif not v[4] then
            v[4] = 'nodeco'
          end
          menu[k] = concat{"<li>",self:link(v[1],lang[v[2]] or v[2],v[3],v[4],nil,v.loading),"</li>"}
        end
      else
        menu[k] = ''
      end
    else  menu[k] = "" -- lang[v[2]] or v[2] or v[1]
    end
  end

  if cli then
    cli.hidden = self.def.hidden and self.def:hidden(nrow,recno)
    self.term:view(cli,recno,dform)
    return
  end

  local links = {}
  local list = self:bind("list")
  local sname = list.model
  for i,row in ipairs(dform.list or {}) do
    local name,frm = row[1],row[2]
    if self:access(name,0,self.user.id,"view") then
      local tab = list:run(name,frm,recno)
      if tab ~= "" then 
         links[#links + 1]=tab
      end
      list.model = sname
    end
  end

  local hidden = ""
  for k,v in pairs(self.hidden and self:hidden(nrow,recno) or {}) do
    hidden = concat{hidden,format("<input type='hidden' name='%s' value='%s'/>",k,v)}
  end
  local action = concat{self.conf.idx,"?lynx=true&mod=",dname,"&recno=",recno}
  local footer = {}
  if guest and dform.footer then
    tins(footer,dform.footer)
  end

  local js = dform.js and concat{"<script>",type(dform.js) == 'string' and dform.js or dform.js(self,nrow) or '',
                         "</script>"} or ""

  return format([[
        <div class="section"><section class="section-space bg"><div class="container">%s
        <div class="debug-info">%s</div>
        <div class="%s">
        <form id="edfrm" method="post" action="%s">
        <input type='hidden' name='fmod' value='%s'/>
        <input type='hidden' name='frecno' value='%s'/>
        %s
        <table width='100%%' class='inner' border='0' cellspacing='0' cellpadding='0'>
          <tr><td valign='top'>
          <table width='100%%' border='0' frame='void' cellpadding='1' cellspacing='1'>
          %s</table>%s
          </td></tr>
          <td valign='top' style="padding-left:2px"></td> 
        </table>%s</form>

        %s<div id="links">%s</div>%s</div></div></section>

        </div>
      ]],
      dform.title or "",
      self.conf.debug and self.user.admin and qry or concat(dbg,"<br/>"),
      dform.class or dname or "",
      action,params.fmod or dname,recno,
      hidden,
      concat(out),
      concat(acts),
      toprn,js,
      concat(links),
      concat(footer)), --frm
      concat(menu),    --acts left menu
      concat(btns,' ') --btns top
end

return db
