------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local util = require "yoke.snap.util"
local concat = table.concat
local insert = table.insert
local format = string.format

local resp = table.class('http.Response')
local cookie = {}

function cookie.opt(name, val)
  if name and val and name ~= "" then
    if type(val == 'boolean') then
      return format("; %s", name)
    end
    return format("; %s=%s", name, val)
  end
  return ""
end

function cookie.make(name, value)
  local opt = {}
  if type(value) == "table" then
    opt = value
    value = value.value
  end
  local out = {concat{name,"=",util.url.encode(value)}}
  if opt.expires then
    insert(out,cookie.opt("expires", os.date("!%A, %d-%b-%Y %H:%M:%S GMT", opt.expires)))
  end
  if opt.maxAge then
    insert(out,cookie.opt("Max-Age", os.date("!%A, %d-%b-%Y %H:%M:%S GMT", opt.maxAge)))
  end
  insert(out,cookie.opt("path", opt.path))
  insert(out,cookie.opt("domain", opt.domain))
  insert(out,cookie.opt("secure", opt.secure))
  insert(out,cookie.opt("HttpOnly", opt.httponly))
  return concat(out)
end

function resp:cookie(name, value)
  local cookies = self.headers["Set-Cookie"]
  if type(cookies) == "string" then
    self.headers["Set-Cookie"] = {cookies, cookie.make(name, value)}
  elseif type(cookies) == "table" then
    insert(self.headers["Set-Cookie"], cookie.make(name, value))
  else
    self.headers["Set-Cookie"] = cookie.make(name, value)
  end
end

function resp:initialize(status, headers)
  resp.status = status or 200
  resp.headers = headers or {}
  if not resp.headers["Content-Type"] then
    resp.headers["Content-Type"] = "text/html"
  end
end

return resp

