------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local menu = {}
local concat,insert = table.concat,table.insert

local cont = "*Press Enter*"
menu.rows = 22
menu.maxlen = 28 --column width

function menu:draw()
  io.write(string.format("%s", self.name))
  local page = 1
  for i,name in ipairs(self[1]) do
    local step = math.ceil(i/menu.rows)
    if page and step ~= page and not menu.back then
      if name:find("\n") then
      io.write(cont)
      if io.read("*l") ~= "" then break end end
      page = step
    end
    if self[2][i] then
      if not (name or ''):find("\n") then io.write("\027[32m")
        io.write(string.format("%2s) %s  ", i, name)) -- y
      else
        io.write(string.format("%2s) ", i))      -- x
        local ii = 0
        for line in string.gmatch(name,"([^\n]*\n)") do
          io.write(string.format("%s",line))
          ii = ii + 1
          step = math.ceil((i+ii)/menu.rows)
          if page and step ~= page and not menu.back then
            io.write(cont)
            page = step
            if io.read("*l") ~= "" then page = nil break end
          end
        end
      end
    end
  end
  io.write("\027[0m")
end

function menu:run()
  self:draw()
  io.write("\n" .. self.prompt)
  while true do
    local line = io.read("*l")
    if line == "" and self.mode == "command" then menu.back = true return false end
    if line == '.' then break end --buy

    local _, _, item,arg,reg = string.find(line or "", "^%s*(%d+)%s*(%d*)%s*([^%s]*)$")
    if self.mode == 'command' and item then
      if arg == "" then arg = item end
      item = tonumber(item)
      arg  = tonumber(arg)

      if self[2][item] then
        return self[2][item](arg,reg)
      end
    end
    io.write(self.prompt)
  end
end

function menu:put(name, action,i)
  if not i then
    insert(self[1],name)
    insert(self[2],action)
  else
    insert(self[1],i,name)
    insert(self[2],i,action)
  end
  return self
end

function menu:del(item)
  if item then
    table.remove(self[1],item)
    table.remove(self[2],item)
  end
end

function menu:add(name, act,i)
  if type(act) == "function" then
    return menu.put(self,name, act,tonumber(i))
  elseif act then
    return menu.put(self,name, function() return act end)
  end
end


function menu:new(name)
  local meta = {}
  function meta:__index(key)
    local func = menu[key]
    if func then
      local cb = function(a, b) return func(self,a, b) end
      self[key] = cb
      return cb
    end
  end
  return setmetatable({
    {}, {},
    name = name and concat{name,"\n"} or "",
  },meta)
end

function menu:loop(prompt)
  self.prompt = prompt or "Number: "
  self.mode = 'command'
  while true do
    local info = self:run()
    if not info then break end
    print(info)
  end
end

function menu:confirm(mes)
  io.write(concat{mes or "Confirm"," (y/n)?: "})
  local ok = io.read("*l")
  if ok ~= 'y' then return false end
  return true
end

function menu:get(info)
  local add = function(line)
    local pos = self.pos
    menu.add(self,string.format("%2s",line),function(id) 
      io.write(self[1][id])
      line = io.read("*L")
      if line ~= "\n" then self[1][id] = line end
      return "Ok"
      end,pos)
    self.pos = self.pos + 1
  end
  if info and info ~= "" then
    for line in info:gmatch("([^\n]+)\n?") do add(concat{line,"\n"}) end
    return "Ok"
  else
    io.write(table.concat{"\027[32m","Type . to end","\027[0m","\n"})
    while true do
      local line = io.read("*L")
      if line ~= ".\n" then
        add(line)
      else
        self.prompt = "Number: "
        self.mode = 'command'
        break
      end
    end
  end
  return "Ok"
end

function menu:ed(info,input)
  local output = {}
  local cnt = 5
  local acts
  acts = function (m)
    m.add(string.format("%2s","Ins"),function(id)
      if id and tonumber(id) > #m[1]-cnt then id = #m[1]-cnt+1  end
      m.pos = id or #m[1]-cnt+1
      m.get() return 'Ok' end)
    m.add(string.format("%2s","Del"),function(id)
      m.mode = 'command'
      if id and tonumber(id) > #m[1]-cnt then id = #m[1]-cnt end
      if id then m.del(id) else m.del(#m[1]-cnt) end
      return "Ok" end)
    m.add(string.format("%2s","Clear"),function()
      m.mode = 'command'
      if self:confirm("Clear all") then 
        m[1] = {} 
        m[2] = {}
        acts(m)
        m.pos = 1
      end 
      return "Ok" end)
    m.add(string.format("%2s","Replace"),function(id,reg)
      m.mode = 'command'
      local from,to = reg:match("([^/]*)/?([^/]*)")
      if from == "" and to == "" then 
        io.write("from:")
        from = io.read("*l")
        if from == "" then return "Ok" end
        io.write("to:")
        to = io.read("*l")
        if to == "" then return "Ok" end
      end
      if id and from ~= "" and to == "" then m[1][id] = from .. "\n"
      elseif id and tonumber(id) > #m[1]-cnt then
        for i=1,#m[1]-cnt do m[1][i] = m[1][i]:gsub(from,to) end
      else m[1][id] = m[1][id]:gsub(from,to) end
      return "Ok" end)
    m.add(string.format("%2s","Write"),function()
      for i=1,#m[1]-cnt do table.insert(output,m[1][i]) end
      return end)
  end

  local m = menu:new(info or "")
  acts(m)
  m.pos = 1
  m.get(input)
  m.loop("Number: ")
  return concat(output)
end

function menu:view(text,mtype,isfile,func)
  local plain = function(html)
    if not html then return "" end
    local cmd  = concat{"echo '", html:gsub("'",'"'),
    "' | (w3m -dump -T text/html 2> /dev/null || lynx -stdin -dump 2> /dev/null || cat )"}
    local fd = io.popen(cmd,"r")
    local text = {}
    if fd then insert(text,fd:read("*a"))
      fd:close()
    end
    return concat(text)
  end

  local show = function(text,func)
    local m = menu:new()
    m.add(text,func or "")
    m.add("Export",function() local fd = io.open("yoke.export.txt","w"); fd:write(text) end)
    m.loop()
  end
  if not isfile then
    show(mtype ~= "html" and text or plain(text),func)
  else
    local cmd = nil
    mtype = mtype or ""
    if string.lower(mtype) == "pdf" then
      cmd = concat{"evince ",text}
      --if not os.execute(cmd) then --todo double run pdf
      --  cmd = concat{"atril ",text}
      --end
    elseif string.find("doc,docx,xls,xlsx,ods,odt",string.lower(mtype)) then
      cmd = concat{"libreoffice ",text}
    elseif string.lower(mtype) == "html" then
      local fd = io.open(text)
      show(plain(fd:read("*a")))
    elseif string.find("png,jpg,jpeg,gif",string.lower(mtype)) then
      cmd = concat{"ristretto ",text}
    elseif string.find("mp4,avi",string.lower(mtype)) then
      cmd = concat{"mpv ",text}
    elseif(menu:confirm("View as text ?")) then
      local fd = io.open(text)
      show(fd:read("*a"))
    end
    if type(cmd) == 'string' then os.execute(cmd) end
  end
  return "Ok"
end

return menu
