------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local socket =  require'socket'
local mimetypes = require 'mimetypes'
local Request = require 'yoke.snap.yard.request'
local Response = require 'yoke.snap.yard.response'
local http = require"yoke.snap.http.init"

local yard = {}
yard.__index = yard

function yard:new(args)
  local serv = {}

  serv.host = args.host or '*'
  serv.port = args.port or '8081'
  serv.location = args.location or ''
  serv.timeout = args.timeout or 0.1

  return setmetatable(serv, self)
end

function yard:run(app)
  local server = assert(socket.bind(self.host, self.port))
  local ip, port = server:getsockname()
  --print(table.concat{'yard on port ',ip,":",port})

  while true do
    local client, errmsg = server:accept()
    if client then
      client:settimeout(self.timeout, 'b')
      local path = self:reply(client, server,app)
      if path == '/pybreak' then break end --todo
    else
      io.stderr:write(table.concat{'Socket accept faild:',errmsg,'\n'})
    end
  end
end

function yard:file(request,response)
  local path = request.path
  if path and self.location ~= '' and path ~= '/' then
    local filename = table.concat{'.',self.location,path}
    local file = io.open(filename, 'rb')
    if file then
      local contype = mimetypes.guess(filename or '') or 'text/html'
      if contype ~= 'text/x-lua' and contype ~= 'text/x-python' then
        response:file(file, contype)
        response.static = true
      else
        response:contype('text/html')
        file:close()
      end
    else
      response:code(404)
    end
  end
end

function yard:env(req)
  local headers = req.headers
  return {
    REQUEST_METHOD = req.method,
    QUERY_STRING = req.query,
    CONTENT_LENGTH = headers['Content-Length'],
    CONTENT_TYPE =  headers['Content-Type'],
    HTTP_COOKIE = headers['Cookie'],
  }
end

function yard:reply(client, server,app)
  local request = Request:new(client, server)
  local response =  Response:new(client)
  local headers = request:header()
  self:file(request,response)

  if app and not response.static and response.status ~= 404 and request.method then
    local env = self:env(request)
    local opt = {
      input = client,
      output = client,
      read = "receive",
      write = "send",
      env = env,
      httpline = true,--todo optimize
    }
    http(app,opt)
    client:close()
  end

  if response.status == 404 then
    response:errmsg(404)
  end
  return request.path
end

return yard
