------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local util = require "yoke.snap.util"
local statuses = require"yoke.snap.http.status"
local concat = table.concat

local errmsg = "Error: $code, $text"
local headline = 'HTTP/1.1 $code $text \r\n'

local Response = {}
Response.__index = Response

function Response:new(client)
  local resp = {}
  resp.headline = ''
  resp.headers = {}
  resp.sent = false
  resp.closed = false
  resp.client = client
  resp.status = 200
  return setmetatable(resp, self)
end

function Response:hdr(key, value)
  self.headers[key] = value
  return self
end

function Response:hdrs(params)
  for key, value in pairs(params) do
    self.headers[key] = value
  end
  return self
end

function Response:contype(value)
  self.headers['Content-Type'] = value
  return self
end

function Response:code(code, text)
  self.status = code
  self.headline = util:sub(headline,{code = code,text = text})
  return self
end

function Response:gethdrs()
  local headers = ''
  for key, value in pairs(self.headers) do
    headers = headers .. key .. ': ' .. value ..'\r\n'
  end
  return headers
end

function Response:errmsg(code)
  self:code(code)
  self:write(util:sub(errmsg,{code = code,text = statuses[code]}), false)
  return self
end

function Response:close()
  self.client:send('\r\n\r\n')
  self.closed = true
end

function Response:sendHeaders(body,chunked)
  if self.sent then
    return self
  end
  if chunked then
    self:hdr('Transfer-Encoding', 'chunked')
  elseif type(body) == 'string' then
    self:hdr('Content-Length', body:len())
  end
  if not self.headers['Content-Type'] then
    self:hdr('Content-Type', 'text/html')
  end

  self.client:send(self.headline .. self:gethdrs())
  self.client:send('\r\n')
  self.sent = true
  return self
end

function Response:write(body, chunked)
  self:sendHeaders(body,chunked)
  self.closed = not(chunked or false)
  if self.closed then
    self.client:send(body)
  elseif #body > 0 then
    self.client:send(concat{util.tohex(#body),'\r\n',body,'\r\n'})
  end

  if self.closed then
    self.client:close()
  end

  return self
end

function Response:file(file, contype)
  self:contype(contype)
  self:code(200)
  local value = file:read('*a')
  file:close()
  self:write(value)
  return self
end

return Response
